__all__ = [
    "__package_name__",
    "__package_url__",
    "__version__",
    "__author__",
    "__author_email__",
    "__description__",
    "__license__",
]

__package_name__ = "workos"

__package_url__ = "https://github.com/workos-inc/workos-python"

__version__ = "0.8.7"

__author__ = "WorkOS"

__author_email__ = "team@workos.com"

__description__ = "WorkOS Python Client"

__license__ = "MIT"
